@@ -846,7 +846,7 @@ impl_tuple!(
 );
 
 #[cfg(feature = "std")]
-mod id_addr_std_derive_impl {
+mod ip_addr_std_derive_impl {
     use crate::BorshSchema as BorshSchemaMacro;
 
     #[derive(BorshSchemaMacro)]
@@ -865,41 +865,41 @@ mod id_addr_std_derive_impl {
     #[borsh(crate = "crate")]
     pub enum IpAddr {
         /// An IPv4 address.
-        V4(std::net::Ipv4Addr),
+        V4(core::net::Ipv4Addr),
         /// An IPv6 address.
-        V6(std::net::Ipv6Addr),
+        V6(core::net::Ipv6Addr),
     }
 }
 
 #[cfg(feature = "std")]
-impl BorshSchema for std::net::Ipv4Addr {
+impl BorshSchema for core::net::Ipv4Addr {
     fn add_definitions_recursively(definitions: &mut BTreeMap<Declaration, Definition>) {
-        <id_addr_std_derive_impl::Ipv4Addr>::add_definitions_recursively(definitions);
+        <ip_addr_std_derive_impl::Ipv4Addr>::add_definitions_recursively(definitions);
     }
 
     fn declaration() -> Declaration {
-        id_addr_std_derive_impl::Ipv4Addr::declaration()
+        ip_addr_std_derive_impl::Ipv4Addr::declaration()
     }
 }
 
 #[cfg(feature = "std")]
-impl BorshSchema for std::net::Ipv6Addr {
+impl BorshSchema for core::net::Ipv6Addr {
     fn add_definitions_recursively(definitions: &mut BTreeMap<Declaration, Definition>) {
-        <id_addr_std_derive_impl::Ipv6Addr>::add_definitions_recursively(definitions);
+        <ip_addr_std_derive_impl::Ipv6Addr>::add_definitions_recursively(definitions);
     }
 
     fn declaration() -> Declaration {
-        id_addr_std_derive_impl::Ipv6Addr::declaration()
+        ip_addr_std_derive_impl::Ipv6Addr::declaration()
     }
 }
 
 #[cfg(feature = "std")]
-impl BorshSchema for std::net::IpAddr {
+impl BorshSchema for core::net::IpAddr {
     fn add_definitions_recursively(definitions: &mut BTreeMap<Declaration, Definition>) {
-        <id_addr_std_derive_impl::IpAddr>::add_definitions_recursively(definitions);
+        <ip_addr_std_derive_impl::IpAddr>::add_definitions_recursively(definitions);
     }
 
     fn declaration() -> Declaration {
-        id_addr_std_derive_impl::IpAddr::declaration()
+        ip_addr_std_derive_impl::IpAddr::declaration()
     }
 }
